import FrequentQueryTemplate from "./FrequentQueryTemplate";

const ITEM_KEY: string = "FrequentQueryData";

class FrequentQueryManager {
  private _id: string = "";
  private _defaultTemplateId: string = "";
  private _defaultTemplateName: string = "";
  private _templateList: Array<FrequentQueryTemplate> = [];

  /**
   * 数据同步到缓存
   */
  private updateData() {
    try {
      let data: any = localStorage.getItem(ITEM_KEY);
      let obj = JSON.parse(data);
      obj[this._id] = this;
      localStorage.setItem(ITEM_KEY, JSON.stringify(obj));
    } catch (e) {
      console.error("更新常用搜索数据出错：", this._id);
    }
  }

  /**
   * 更新模板名称
   * @param templateItem
   * @param name
   */
  private updateTemplateName(
    templateItem: FrequentQueryTemplate,
    name: string
  ) {
    templateItem.name = name;
  }



  /**
   * 更新模板是否自动查询
   * @param templateItem
   * @param isAutoSearch
   */
  private updateTemplateAutoSearch(
    templateItem: FrequentQueryTemplate,
    isAutoSearch: Boolean
  ) {
    templateItem.isAutoSeach = isAutoSearch;
  }

  /**
   * 根据默认模板id同步默认模板名称
   */
  private updateDefaultTemplateName() {
    this._defaultTemplateName = "";
    if (this._defaultTemplateId) {
      let defaultTemplate = this.getDefaultTemplate();
      this._defaultTemplateName = defaultTemplate ? defaultTemplate.name : "";
    }
  }

  /**
   * 根据下标获取模板
   * @param index
   */
  getTemplateByIndex(index: number) {
    return this._templateList[index];
  }

  /**
   * 根据id获取模板
   * @param id
   */
  getTemplateById(id: string) {
    for (let i = 0; i < this._templateList.length; i++) {
      let temp: FrequentQueryTemplate = this._templateList[i];
      if (temp.id === id) {
        return temp;
      }
    }

    return null;
  }

  /**
   * 获取所有模板
   *
   */
  getAllTemplates() {
    return this._templateList;
  }

  /**
   * 添加模板
   * @param key
   * @param name
   * @param templateData
   * @param isAutoSeach
   */
  addTemplate(name: string, templateData: any, isAutoSeach: Boolean = false) {
    if (!name) {
      return { res: -1, msg: "模板名字不能为空" };
    }
    let isExistName = this._templateList.some(item => {
      return item.name == name;
    });
    if (isExistName) {
      return { res: -2, msg: "模板名字已存在" };
    }

    let item: FrequentQueryTemplate = new FrequentQueryTemplate(
      name,
      templateData,
      isAutoSeach
    );
    this._templateList.push(item);
    this.updateData();
    return { res: 1, item: item };
  }

  /**
   * 根据下标删除模板
   * @param index
   */
  removeTemplateByIndex(index) {
    this._templateList.splice(index, 1);
    this.updateData();
  }

  /**
   *
   * @param id 根据id删除模板
   */
  removeTemplateById(id: string) {
    for (let i = 0; i < this._templateList.length; i++) {
      let temp: FrequentQueryTemplate = this._templateList[i];
      if (temp.id === id) {
        this.removeTemplateByIndex(i);
        return;
      }
    }
  }

  /**
   * 是否设置为默认模板
   * @param templateId
   */
  isDefaultTemplate(templateId: string) {
    if (templateId === this._defaultTemplateId) {
      return true;
    }
    return false;
  }

  /**
   * 获取默认模板，若无设置默认模板，返回null
   */
  getDefaultTemplate() {
    let templateItem = this._templateList.find(item => {
      return this.isDefaultTemplate(item.id);
    });
    return templateItem;
  }

  /**
   * 更新模板
   * @param templateItem
   * @param name 模板名称
   * @param templateData  模板数据
   * @param isAutoSearch  是否自动搜索
   */
  updateTemplate(
    templateItem: FrequentQueryTemplate,
    name: string,
    templateData: any,
    isAutoSearch: Boolean
  ) {
    if (arguments.length >= 2 && name) {
      let isExistName = this._templateList.some(item => {
        return item.name == name && item.id != templateItem.id;
      });
      if (isExistName) {
        return { res: -1, msg: "模板名字已存在" };
      }
      this.updateTemplateName(templateItem, name);
    }
    if (arguments.length >= 3 && templateData) {
      // this.updateTemplateData(templateItem, templateData);
    }
    if (
      arguments.length >= 4 &&
      isAutoSearch !== null &&
      isAutoSearch !== undefined
    ) {
      this.updateTemplateAutoSearch(templateItem, isAutoSearch);
    }
    this.updateData();
    return { res: 1 };
  }


  public get defaultTemplateName(): string {
    return this._defaultTemplateName;
  }

  public get defaultTemplateId(): string {
    return this._defaultTemplateId;
  }
  public set defaultTemplateId(value: string) {
    this._defaultTemplateId = value;
    this.updateDefaultTemplateName();
    this.updateData();
  }

  constructor(
    key: string,
    defaultTemplateId: string = "",
    list: Array<FrequentQueryTemplate> = []
  ) {
    this._id = key;
    this._templateList = list;
    this._defaultTemplateId = defaultTemplateId;
    this.updateDefaultTemplateName();
  }
}

function getFrequentQueryInstance(key: string) {
  if (!key) {
    return null;
  }
  let data: any = localStorage.getItem(ITEM_KEY);
  if (!data) {
    let fqm = new FrequentQueryManager(key);
    let obj = {};
    obj[key] = fqm;
    let temp = JSON.stringify(obj);

    localStorage.setItem(ITEM_KEY, temp);
    // fqm.addTemplate("TEST1", "TEST1", { a: 1, b: 2 }, true);
    // fqm.addTemplate("TEST2", "TEST2", { a: 1, b: 2 }, false);
    return fqm;
  } else {
    let obj = JSON.parse(data);
    if (obj.hasOwnProperty(key)) {
      let itemObj = obj[key];
      let list = [];
      let defaultTemplateId = "";
      itemObj._templateList.forEach(templateEle => {
        let templateItem = new FrequentQueryTemplate(
          templateEle._name,
          templateEle._templateData,
          templateEle._isAutoSeach
        );
        list.push(templateItem);
        if (templateItem.name == itemObj._defaultTemplateName) {
          defaultTemplateId = templateItem.id;
        }
      });

      let fqm = new FrequentQueryManager(key, defaultTemplateId, list);
      return fqm;
    }else{
      let fqm = new FrequentQueryManager(key, "", []);
      return fqm;
    }
  }
  return null;
}

export { FrequentQueryManager, getFrequentQueryInstance };
